/*
 * File:		CreateShape_0_1.jsx
 * Author:		Peter Torpey
 * 				www.petertorpey.com
 * Version:		0.1		06-25-05
 *     		   	Original code Copyright 2005 Peter Torpey.
 *      	   	This code may be used, distributed, and modified for personal
 *      	   	use provided the name and URL of the original author, above,
 *       	  	remain intact.
 *
 * 				If this code is used in the creation of a notable production,
 * 				please let the author know. The author's e-mail address may be
 * 				found at http://www.petertorpey.com/contact.html .
 *
 * Host:		Adobe After Effects Professional 6.5
 *
 *
 * Dependencies:
 * 	(lib)/UILayout_13.jsx
 *
 *
 * Installation:
 * 	Place this file in After Effects' "Script" folder or a subfolder.
 * 	On Windows, the default location is:
 * 		C:\Program Files\Adobe\After Effects 6.5\Support Files\Scripts\
 *
 * Description:
 *  Creates a mask for the selected layer which is a regular convex polygon
 * 		with optional rounding and stellation.
 *
 *
 * Usage:
 * 	Run this script with one or more layers selected to create a mask shape
 * 		for each.
 *
 * 	Parameters:
 * 		Sides / Points:
 * 			This is the number of sides (or vertices) in the polygon. For
 * 			stars, this value will be the number of points in the star.
 * 		Stellate:
 * 			Enables the creation of a star-shaped polygon
 * 		Star Inner Radius:
 * 			This value is a percentage of the overall radius of the star.
 * 		Rounding:
 * 			A percentage value indicating how round the polygon should be
 * 				Increasing the rounding value increases the curvature of the
 * 				edges.
 * 		Lock Height to Width:
 * 			When checked, the height value will be the same as the width value
 * 			and interpreted as pixels or percentages as the width value.
 * 		Width and Height:
 * 			These fields define the outermost dimension of the shape. Values
 * 				may be in pixels (eg. 100px) or a percentage of that dimension
 * 				(eg. 30%).
 * 		Rotation:
 * 			The overal all rotation of the shape in degrees.
 *
 *
 * Notes:
 *  This script and any scripts upon which it depends have been tested under
 *  	AE6.5 on WinXP.
 *
 *
 * Known Issues:
 *  None.
 *
 *
 * For the Future:
 *  Corner rounding.
 * 	Inner radius rotation.
 *  Apply to selected keyframes, if any (like ReversePath_0_1).
 *
 *
 */

	/*** APPLICATION GLOBALS ***/


{

	/*** DEFAULTS ***/
	var NAME_STR = "Create Shape";

	var D_WIDTH_HEIGHT = "300px";
	var D_ROUNDING = "0%";
	var D_STAR_RAD = "50%";
	var D_DO_STAR = false;
	var D_VERTS = 5;
	var D_LOCK_WIDTH_HEIGHT = true;

	/*** SCRIPT GLOBALS ***/
	var readyState = true;

	// includes
	var includeScripts = ["(lib)/UILayout_13.jsx"];
	for (var i = 0; i < includeScripts.length; i++) {
		var scriptFile = new File(includeScripts[i]);
		if (scriptFile.open()) {
			eval(scriptFile.read());
			scriptFile.close();
		}
		else {
			readyState = false;
			alert(NAME_STR + ":\n\nUnable to open included file: \"" + includeScripts[i] + "\".");
		}
	}

	function parseDistance(str, ref) {
		var ps = str.match(/(\d*(?:\.\d*)?)\s*(%|px|pix)?/);
		if (ps) {
			if (ps[2] == "%") {
				return parseFloat(str) / 100 * ref;
			}
			else {
				return parseFloat(str);
			}
		}

		return parseFloat(ref);
	}

	function buildDialog() {
		var dlg = new Window("dialog", NAME_STR, [100, 100, 470, 500]);

		var shpPnl = dlg.addUI("panel", "Shape Properties");
		var vertsField = shpPnl.addUI("labeledittext", "Sides / Points");
		vertsField.layoutFill = UILM_FILL_EXPAND;
		vertsField.text = D_VERTS;
		var starCheck = shpPnl.addUI("checkbox", "Stellate");
		starCheck.layoutFill = UILM_FILL_SHRINK;
		starCheck.value = D_DO_STAR;
		var starRadField = shpPnl.addUI("labeledittext", "Star Inner Radius");
		starRadField.layoutFill = UILM_FILL_EXPAND;
		starRadField.enabled = false;
		starRadField.text = D_STAR_RAD;
//		shpPnl.addUI("break");
		var rndField = shpPnl.addUI("labeledittext", "Rounding");
//		rndField.layoutFill = UILM_FILL_EXPAND;
		rndField.text = D_ROUNDING;

		var posPnl = dlg.addUI("panel", "Size & Rotation");
		var lockCheck = posPnl.addUI("checkbox", "Lock Height to Width");
		lockCheck.value = D_LOCK_WIDTH_HEIGHT;
		var widthField = posPnl.addUI("labeledittext", "Width");
		widthField.text = D_WIDTH_HEIGHT;
		var heightField = posPnl.addUI("labeledittext", "Height");
		heightField.enabled = false;
		heightField.text = D_WIDTH_HEIGHT;
		posPnl.addUI("break");
		var rotField = posPnl.addUI("labeledittext", "Rotation");
		rotField.text = 0;

		dlg.addUI("button", "Cancel", {name: "cancel"});
		dlg.addUI("button", "Create Shape", {name: "ok"});

		// handlers
		vertsField.onChange = function () {
			var verts = parseInt(vertsField.text);
			if (!(verts && verts >= 3)) {
				verts = 3;
				alert("The number of vertices for the shape must be 3 or greater.");
			}
			vertsField.text = verts;
		};

		rndField.onChange = function () {
			var rnd = parseFloat(rndField.text);
			if (!(rnd && (rnd >= 0 && rnd <= 100))) {
				rnd = 0;
				alert("The Rounding value must be in the range 0% to 100%.");
			}
			rndField.text = rnd + "%";
		};

		widthField.onChange = function () {
			if (lockCheck.value) {
				heightField.text = widthField.text;
			}
		};

		lockCheck.onClick = function () {
			heightField.enabled = !lockCheck.value;
		};

		starCheck.onClick = function () {
			starRadField.enabled = starCheck.value;
		};

		// accessors (Yes, I'm using the UI as the model, today...)
		dlg.getVertices = function () {
			return parseInt(vertsField.text);
		};

		dlg.getRounding = function () {
			var rad = parseFloat(rndField.text);
			if (rad < 0.0) { rad = 0.0; }
			return rad / 100.0;
		};

		dlg.getWidth = function (ref) {
			return parseDistance(widthField.text, ref);
		};

		dlg.getHeight = function (ref) {
			return parseDistance(heightField.text, ref);
		};

		dlg.getRotation = function () {
			var rot = parseFloat(rotField.text);
			if (rot) {
				while (rot > 360.0) {
					rot -= 360.0;
				}
				while (rot < 0) {
					rot += 360.0;
				}
				return rot;
			}
			return 0.0;
		};

		dlg.getDoStar = function () {
			return starCheck.value;
		};

		dlg.getStarRad = function () {
			var rad = parseFloat(starRadField.text);
			if (rad < 0.0) { return rad = 0.0; }
			return rad / 100.0;
		};

		layoutUI(dlg);
		dlg.center();
		return dlg;
	}

	function createShape(sides, cX, cY, width, height, rotation, rounding, starRad) {
		var shape = new Shape();
		var tVerts = new Array()
		var tInTans = new Array();
		var tOutTans = new Array()
		var vert;

		var doRound = (rounding > 0);

		if (starRad && starRad >= 0) {
			sides *= 2;
		}

		var angle = (2 * Math.PI) / sides;

		// convert deg to rad
		var rot = (360.0 / (2 * Math.PI)) * rotation;
		var theta = angle + rot - Math.PI / 2.0;

		width /= 2.0;
		height /= 2.0;

		shape.closed = true;

		if (starRad && starRad >= 0) {
			var inWidth = starRad * width;
			var inHeight = starRad * height;
			for (var i = 0; i < sides; i++) {
				if (i % 2) {
					vert = [width * Math.cos(theta), height * Math.sin(theta)];
					if (doRound)  {
						tInTans.push([vert[1] * rounding, -vert[0] * rounding]);
						tOutTans.push([-vert[1] * rounding, vert[0] * rounding]);
					}
					else {
						tInTans.push([0, 0]);
						tOutTans.push([0, 0]);
					}
					vert[0] += cX;
					vert[1] += cY;
					tVerts.push(vert);
				}
				else {
					tVerts.push([cX + inWidth * Math.cos(theta), cY + inHeight * Math.sin(theta)]);
					tInTans.push([0, 0]);
					tOutTans.push([0, 0]);
				}
				theta += angle;
			}
		}
		else {
			for (var i = 0; i < sides; i++) {
				vert = [width * Math.cos(theta), height * Math.sin(theta)];
				theta += angle;
				if (doRound)  {
					tInTans.push([vert[1] * rounding, -vert[0] * rounding]);
					tOutTans.push([-vert[1] * rounding, vert[0] * rounding]);
				}
				vert[0] += cX;
				vert[1] += cY;
				tVerts.push(vert);
			}
		}
		shape.vertices = tVerts;
		shape.inTangents = tInTans;
		shape.outTangents = tOutTans;
		return shape;
	}

	function main() {
		if (!readyState) { return -1; }
		var curItem = app.project.activeItem;
		if (curItem != null && (curItem instanceof CompItem)) {
			var curLayers = curItem.selectedLayers;
			if (curLayers.length > 0) {
				var curLayer, curMasks, curMask, shape;
				var dialog = buildDialog();
				if (dialog.show() == 1) {
					app.beginUndoGroup(NAME_STR);
					for (var i = 0; i < curLayers.length; i++) {
						curLayer = curLayers[i];
						shape = createShape(dialog.getVertices(),
											curLayer.anchorPoint.value[0],
											curLayer.anchorPoint.value[1],
											dialog.getWidth(curLayer.width),
											dialog.getHeight(curLayer.height),
											dialog.getRotation(),
											dialog.getRounding(),
											(dialog.getDoStar()) ? dialog.getStarRad() : -1);
						curMasks = curLayer.property("ADBE Mask Parade");
						if (curMasks && curMasks.canAddProperty("ADBE Mask Atom")) {
							curMask = curMasks.addProperty("ADBE Mask Atom");
							curMask.property("ADBE Mask Shape").setValue(shape);
						}
					}
					app.endUndoGroup();
				}
			}
			else {
				alert("Please select a layer on which to apply the created mask shape.");
			}
		}
		else {
			alert("Please select a layer on which to apply the created mask shape.");
		}
	}

	main();

}
